procedure AptAstrom (images)

# Adds astrometry information to APT image files. The files are
# presumed to be full 2x3 degree images. The process takes 1 second 
# per image using a 450MHz Pentium III.
#
# The steps taken are:
#
#   (1) Search the SAO catalog for all stars within a reasonable  
#       distance of the image centre.
#
#   (2) Call "astxymatch.e" to locate the stars within the image.
#
#   (3) Perform two iterations of "ccmap" to calculate the plate
#       constants, eliminating stars that have large errors.
#
#   (4) Write the plate constants to the image header. For example,
#
#   RADECSYS= 'FK5     '
#   WCSDIM  =                    2
#   CTYPE1  = 'RA---TAN'
#   CTYPE2  = 'DEC--TAN'
#   CD1_1   =  2.61084059946544E-5
#   CD1_2   =  0.00261515960979689
#   CD2_1   =  -0.0026142538735568
#   CD2_2   =  1.38221595297264E-5
#   LTM1_1  =                   1.
#   LTM2_2  =                   1.
#   WAT0_001= 'system=image'
#   WAT1_001= 'wtype=tan axtype=ra'
#   WAT2_001= 'wtype=tan axtype=dec'
#
#   (5) Produce a one-line text summary of the number of stars located,
#       the plate scale in arcseconds/pixel in X & Y, the rotation
#       angle of the X,Y axes, and the residual errors in arcseconds. 
#       Here is an example of the output format.
#
# im29478.fits:
#     22 stars; scale: 9.412 9.415; rot: 89.428 89.697; err: 3.27 0.606
#
# Note that the residual errors are often quite large, perhaps several
# arcseconds (one-third of an APT pixel). This is due to the difficulty
# in centroiding the SAO stars, which are often saturated.
#
# Ideally we should use a catalog of fainter stars, perhaps mag 11,
# sampled to uniformly cover the sky. About 250,000 stars would be
# about right.
#
# task AptAstrom = apt$AptAstrom.cl
#
# Michael Ashley / UNSW / 27-Apr-1999

string images="" {prompt="Input images"}

char   version = "27Apr1999"  {prompt="Program version"}

struct *imglist

begin
  string Iimages
  string img, shfile, shfile2, imgfile
  string AptBin
  real ra, dec, DeltaRa, DeltaDec
  int lines

  unlearn ccmap
  unlearn ccfind
  unlearn aptxymatch

  Iimages = images

  shfile  = mktemp ("/tmp/mcba")
  shfile2 = mktemp ("/tmp/mcba")
  imgfile = mktemp ("/tmp/mcba")

  AptBin = envget("apt") // "bin" // envget("arch") // "/"

  sections (Iimages, option="fullname", > imgfile)

  imglist = imgfile
  while (fscan (imglist, img) != EOF) {

  imgets (image = img, param = "RA")
  ra = real(imgets.value)
  ra *= 15.0 * 3.14159265359 / 180.0

  imgets (image = img, param = "DEC")
  dec = real(imgets.value)
  dec *= 3.14159265359 / 180.0

# We make the area a bit larger, to allow for errors in the position of
# the centre.

  DeltaRa  = 1.10 * (3.0 * 0.5 * 3.14159265359 / 180.0) / cos(dec)
  DeltaDec = 1.10 * 2.0 * 0.5 * 3.14159265359 / 180.0

  delete (shfile, ver-, >& "dev$null")
  printf (AptBin // "sao %f %f %f %f > junk.1\n", ra, dec, DeltaRa, DeltaDec,  >> shfile)
  sh (shfile)

  delete ("out.txt",  ver-, >& "dev$null")
  delete ("out1.txt", ver-, >& "dev$null")
  delete ("out2.txt", ver-, >& "dev$null")

  aptxymatch (image=img, refstars="junk.1", output="out.txt")

  ccmap (input="out.txt",database="dbase.txt", image=img, xc=1, yc=2,
         lngc=3, latc=4, lngunit="rad", latunit="rad", project="tan", 
         fit="general", funct="polynomial", xxo=2, yyo=2, xxt="half", 
         yxo=2, yyo=2, yxt="half", reject=1.5, update+, interact-, verbose-)

  ccfind (input="out.txt", output="out1.txt", image=img, lngc=3, latc=4,
         lngu="rad", latu="rad", usewcs+, sbox=11, cbox=9, maxit=2, 
         verbose-, >& "dev$null")

  wc out1.txt | scan (lines)
  lines = lines - 9
  printf ("%s:\n%6d stars; ", img, lines)

  ccmap (input="out1.txt",database="dbase.txt", image=img, xc=6, yc=7,
         lngc=3, latc=4, lngunit="rad", latunit="rad", project="tan", 
         fit="general", funct="polynomial", xxo=2, yyo=2, xxt="half", 
         yxo=2, yyo=2, yxt="half", reject=1.2, update+, interact-, verbose+,
         >& "out2.txt") 

  delete (shfile2, ver-, >& "dev$null")
  printf (AptBin // "AptAstrom1.pl < out2.txt", >> shfile2)
  sh (shfile2)
}
#  starfind (image = img, output = "junk.2", hwhmpsf = 0.85, 
#   roundlo = 0.0, roundhi = 1.0, sharplo = 0.7, sharphi = 1.4,
#   threshold = 20000.0,  fradius = 2.5, sepmin = 5.0, npixmin = 5)
#   
end
