procedure generate_matrix (images, output, xcen, ycen, boxsize, shifts, 
                           verbose)
#
# Michael Ashley / UNSW / 29-Apr-1998
#
# This task will take a list of "images" and produce a 2D image "output"
# which results from tiling a subset of each image centred on
# (xcen, ycen) as modified by an (x,y) shift obtained from the
# "shifts" file. Each subset is a square of side "boxsize".
#
# The images are tiled from the bottom left corner, with the column 
# increasing most rapidly.
#
# The dimensions of the output image are chosen automatically, to make
# it as close to square as possible.
#
# Each subset has a zero-point subtracted before being placed
# into the output image. The zero-point is the minimum pixel in the subset.
#
# Type the following command into the IRAF cl to use this program:
#
# task generate_matrix=apt$generate_matrix.cl
#

string images=""         {prompt="Images to mosaic"}
string output=""         {prompt="Output image"}
real   xcen              {prompt="X coord of first image to place in centre"}
real   ycen              {prompt="Y coord of first image to place in centre"}
int    boxsize           {prompt="Size of box extracted from each image"}
string shifts=""         {prompt="File containing offsets of images"} 
bool   verbose=no        {prompt="Verbose output?"}
char   version = "30Jul1999"  {prompt="Program version"}

struct *IMGLIST
struct *SHIFTS

begin

# Copies of the parameters.

   string Iimages, Ioutput, Ishifts
   real   Ixcen, Iycen
   int    Iboxsize
   bool   Iverbose

# Local variables.

   int    nimg, nx, ny, xpix, ypix, nrows
   int    x_lo, y_lo, X_lo, Y_lo
   int    x_hi, y_hi, X_hi, Y_hi
   int    xdim, ydim, x_num, y_num
   real   x_shift, y_shift, min_pix
   bool   first
   string imgfile, tmpimg1, tmpimg2, img

# Assign positional parameters to local variables.

   Iimages   = images
   Ioutput   = output
   Ixcen     = xcen
   Iycen     = ycen
   Iboxsize  = boxsize
   Ishifts   = shifts
   Iverbose  = verbose

# Restore defaults for all the IRAF routines we use.

   unlearn sections
   unlearn imexpr
   unlearn imdel
   unlearn imstatistics
   unlearn imarith
   unlearn imcopy

   imcopy.verbose  = Iverbose
   imexpr.verbose  = Iverbose
   imarith.verbose = Iverbose

   first = yes

# Form temporary filenames.

   imgfile =  mktemp ("/tmp/mcba")
   tmpimg1 =  mktemp ("/tmp/mcba")
   tmpimg2 =  mktemp ("/tmp/mcba")

# Check existence of input @file.

   if ( (stridx("@",Iimages) == 1) &&
        (!access( substr(Iimages,2,strlen(Iimages))) ) ) {
      print ("GENERATE_MATRIX: Input image @file ",Iimages," does not exist",
         " - processing aborted.")
      beep ; goto cleanup
   }

# Expand input template into file lists.

   sections (Iimages, option="fullname", > imgfile)
   nimg = sections.nimages
   if (nimg == 0) {
      print ("GENERATE_MATRIX: No images in input list - processing aborted.")
      beep ; goto cleanup
   }
   IMGLIST = imgfile

# Check existence of shifts file.

   if (!access(Ishifts)) {
      print ("GENERATE_MATRIX: Can't access shifts file.")
      beep ; goto cleanup
   }
        
   SHIFTS = Ishifts

# Determine the optimal image size.

   nx = int(sqrt(nimg))
   ny = int((nimg - 0.5) / nx) + 1

   xpix = nx * Iboxsize
   ypix = ny * Iboxsize
      
# Create a zero-filled output image of the correct size.

   imdel (Ioutput, >& "dev$null")
   imexpr (expr="repl(0,"//xpix//")", output=Ioutput, 
           dims=xpix//","//ypix)

# Scan through the input images.

   x_num = 0
   y_num = 0

   while (fscan (IMGLIST,img) != EOF) {
      if ( !access (img) && !access (img//".fits") && !access (img//".imh") ) {
         print ("GENERATE_MATRIX: Image ",img," does not exist")
         beep ; goto cleanup
      } else {
#         if (stridx('.',img) != 0) {
#            img = substr(img,1,stridx('.',img)-1)
#         }

         imgets (image = img, param = "i_naxis1")
         xdim = int (imgets.value)
         imgets (image = img, param = "i_naxis2")
         ydim = int (imgets.value)

# Find the (x,y) shift for the current image.

         if (fscan(SHIFTS, x_shift, y_shift) != EOF) {

# Find the lower and upper bounds of the desired section.

            x_lo = min(xdim, max(1, int(Ixcen - x_shift + 0.5 - Iboxsize / 2)))
            y_lo = min(ydim, max(1, int(Iycen - y_shift + 0.5 - Iboxsize / 2)))
            x_hi = min(xdim, x_lo + Iboxsize - 1)
            y_hi = min(ydim, y_lo + Iboxsize - 1)

# Find where to put the section in the output image.

            X_lo = 1 + x_num * Iboxsize
            X_hi = X_lo + (x_hi - x_lo)
            Y_lo = 1 + y_num * Iboxsize
            Y_hi = Y_lo + (y_hi - y_lo)

# Extract the subset to a temporary file.

            imdel (images=tmpimg1//","//tmpimg2, >& "dev$null")
            imcopy (input=img//"["//x_lo//":"//x_hi//","//y_lo//":"//y_hi//"]",
              output=tmpimg1)

# Find the minimum pixel, and subtract it.

            imstatistics (images=imgimg1,
              fields="min",format-) | scan (min_pix)

            imarith (operand1 = tmpimg1, op = "-", 
               operand2 = min_pix, result = tmpimg2)
 
# Place the subset into the output image.

            imcopy (input=tmpimg2,
               output=Ioutput//"["//X_lo//":"//X_hi//","//Y_lo//":"//Y_hi//"]")

# Increment the raster position of the subset in the output image.

            x_num += 1
            if (x_num == nx) {
              x_num = 0
              y_num += 1
            }
         } else {
            print ("GENERATE_MATRIX: reached premature EOF on shifts file")
            beep ; goto cleanup
         }
      }
   }

cleanup:

# Delete temporary files

   SHIFTS = ""
   delete (imgfile, ver-, >& "dev$null")
   imdel (images=tmpimg1//","//tmpimg2, >& "dev$null")

end
