procedure initial_shifts (images, shifts, disp_each, verbose)

#
# Michael Ashley / UNSW / 29-Apr-1998
#
# Given a set of input "images", this program will produce a set of
# (x,y) offsets in the file "shifts", based on centroiding the same
# star in each image. The star is chosen interactively using the
# image display and cursor. If "disp_each" is true, then each image
# in the set is displayed, and the user can manually select the same
# star (this choice is appropriate if large offsets can occur from
# one image to the next). If "disp_each" is false, only the first
# image is displayed for user-interaction, and the initial (x,y) guess 
# for the star position is based on the centroid from the previous
# image (this works well for long time-series images where only slow
# drift has occurred).
#
# Type the following command into the IRAF cl to use this program:
#
# task initial_shifts = apt$initial_shifts.cl
#
# If the cursor reading is not working properly, try doing "=imcur" a few
# times at the cl prompt.

string images=""         {prompt="Images to align"}
string shifts=""         {prompt="Output file to contain the image shifts"} 
bool   disp_each=yes     {prompt="Display each image?"}
bool   verbose=no        {prompt="Verbose output?"}
char   version = "30Jul1999"  {prompt="Program version"}

struct *IMGLIST

begin

# Copies of the parameters.

   string Iimages, Ishifts
   bool   Idisp_each, Iverbose

# Local variables.

   string imgfile, tmpfile, img, p1, p2, p3
   int    nimg, cbox
   bool   first
   real   x, y, newx, newy, x_zero, y_zero
   int    wcs, ix, iy
   struct command
   string key
   imcur  cursor

# Assign positional parameters to local variables.

   Iimages       = images
   Ishifts       = shifts
   Idisp_each    = disp_each
   Iverbose      = verbose

# Restore defaults for all the IRAF routines we use.

   unlearn display
   unlearn imcntr
   unlearn sections
   unlearn generate_matrix

# Form temporary filenames.

   imgfile  =  mktemp ("/tmp/mcba")
   tmpfile  =  mktemp ("/tmp/mcba")

   first = yes

# Check existence of input @file.

   if ( (stridx("@",Iimages) == 1) &&
        (!access( substr(Iimages,2,strlen(Iimages))) ) ) {
      print ("INITIAL_SHIFTS: Input image @file ",Iimages," does not exist",
         " - processing aborted.")
      beep ; goto cleanup
   }

# Expand input template into file lists.

   sections (Iimages, option="fullname", > imgfile)
   nimg = sections.nimages
   if (nimg == 0) {
      print ("INITIAL_SHIFTS: No images in input list - processing aborted.")
      beep ; goto cleanup
   }
   IMGLIST = imgfile

# Choose the size of the centroiding box. We use a larger one when the
# user is not manually positioning on each star.

   if (Idisp_each) {
     cbox = 5
   } else {
     cbox = 15
   }

# Delete the existing "shifts" file, if any.

   delete (Ishifts, ver-, >& "dev$null")

# Scan through each input image, using the first one as the reference.

   while (fscan (IMGLIST,img) != EOF) {
      if ( !access (img) && !access (img//".fits") && !access (img//".imh") ) {
         print ("INITIAL_SHIFTS: Image ",img," does not exist")
         beep ; goto cleanup
      } else {
#         if (stridx('.',img) != 0) {
#            img = substr(img,1,stridx('.',img)-1)
#         }

# Ask the user to identify a nice-looking star, or use the
# centroid from the previous image if Idisp_each is false.

         if (first) {
           printf ("Displaying the first image...\n");
           printf ("Position the cursor on one star that will be used to align the images.\n");
         }
         if (first || Idisp_each) {
           display (image=img, frame=1, zscale+, contrast=0.25, zrange-)

           printf ("Press a key to mark the star; ")
           if (fscan (imcur, x, y, wcs, command) == EOF) {
             printf ("INITIAL_SHIFTS: hmm... EOF on cursor read; try typing =imcur\n")
             beep; goto cleanup
           }
         }

# Centroid the approximate coordinates given by the user.

         imcntr (input=img, x_init=x, y_init=y, cboxsize=cbox) | 
              scan (p1, p2, x, p3, y)

# If this is the first image, we use this coordinate as a zero point
# for calculating offsets of subsequent images.

         if (first) {
           x_zero = x
           y_zero = y
           first = no
         }

# Print out info, calculate the shifts, and send the results to the
# screen and the Ishifts file.

         printf ("coord = (%.2f,%.2f);", x, y)
         newx = x_zero - x
         newy = y_zero - y
         printf (" offset = (%.2f, %.2f)\n", newx, newy)
         printf ("%9.2f %9.2f %s\n", newx, newy, img, >> Ishifts)
      }
   }

# Make an image matrix containing all the centroided stars, this is
# to allow a quick-look to see if the overall process was successful.

   printf ("generating quick-look image matrix, please wait...\n");
   generate_matrix (images=Iimages, output=tmpfile, 
                    xcen=x_zero, ycen=y_zero, boxsize=30, shifts=Ishifts, 
                    verbose=Iverbose)

   display (image=tmpfile, frame=2, zscale+, contrast=0.25, zrange-)

cleanup:

# Delete temporary files

   IMGLIST = ""
   delete (imgfile, ver-, >& "dev$null")
   delete (tmpfile, ver-, >& "dev$null")

end
