procedure AptReduce ()

# task AptReduce = home$apt/AptReduce.cl

# An IRAF program to process APT images.

# This works for 800x1200 pixel images, taken with "gain lo speed lo".
# The images must have names like im?????.fits.  Put all the images in
# a directory somewhere. cd to the directory and run AptProc. Four
# text files will be created:
#
#     Images     - a list of all the im?????.fits images
#     Flats      - 800x1200 images that appear to be flat fields on the
#                  basis of their statistics
#     Twilights  - 800x1200 images that have IMAGETYP = 'twilight'
#     Stars      - all other 800x1200 images
#     Unknowns   - all other images
#
# The program then creates a SuperFlat image from the Flats,
# and uses it to flatten the images in the Stars file.
#
# If files of the above names already exist, their contents is assumed to
# be correct, and they are not regenerated. So, if you want to redo the
# flat fields after removing some dubious frames, simply edit the Flats
# file, delete SuperFlat, and re-run AptReduce.
#
# To create a file anew, simply delete it and rerun AptReduce.
#
# To do:
#   - add processing of the twilight flats 
#   - handle errors like no flat fields
#
# Michael Ashley / UNSW / 27-Sep-1998

char   version = "27Sep98"  {prompt="Program version"}

struct *imglist

begin
  real   mean, stddev
  string ImageFile, StarFile, FlatFile, TwilightFile, UnknownFile, Flat, SuperFlat
  string Twilight, img, imagetyp, t1, t2, t3
  int    n1, n2, i, NumTwilights
  real   pmin, pmax, percent
  bool   HaveImages
  bool   HaveStars
  bool   HaveFlats
  bool   HaveTwilights
  bool   HaveUnknowns
  bool   HaveFlat
  bool   HaveSuperFlat
  bool   HaveTwilight

# Load the parameters into local variables so that they are prompted
# for early on, and in the right order.


# Restore defaults for all the IRAF routines we use.

  unlearn AptProc
  unlearn sections
  unlearn imcopy
  unlearn imcombine
  unlearn imdelete 
  unlearn imstatistics 
  unlearn imarith

  ImageFile     = "Images"
  StarFile      = "Stars"
  FlatFile      = "Flats"
  TwilightFile  = "Twilights"
  UnknownFile   = "Unknowns"
  Flat          = "Flat"
  SuperFlat     = "SuperFlat"
  Twilight      = "Twilight"

  HaveImages     = access(ImageFile)
  HaveStars      = access(StarFile)
  HaveFlats      = access(FlatFile)
  HaveTwilights  = access(TwilightFile)
  HaveUnknowns   = access(UnknownFile)

  HaveFlat      = access(Flat//".imh") || access(Flat//".fits")
  HaveSuperFlat  = access(SuperFlat//".imh") || access(SuperFlat//".fits")
  HaveTwilight  = access(Twilight//".imh") || 
                       access(Twilight//".fits")

  if (!HaveImages) {
    printf ("finding images...")
    sections ("im?????.fits", option="fullname", > ImageFile)
  }

  if (!HaveStars) {
     printf ("", > StarFile)
  }

  if (!HaveFlats) {
     printf ("", > FlatFile)
  }

  if (!HaveTwilights) {
     printf ("", > TwilightFile)
  }

  if (!HaveUnknowns) {
     printf ("", > UnknownFile)
  }

  imglist = ImageFile
  if (!HaveStars || !HaveFlats || !HaveTwilights || !HaveUnknowns) {
    printf ("identifying images...")
    while (fscan (imglist, img) != EOF) {
      i = strlen (img)
      if (substr (img, i-4, i) == ".fits") {
        img = substr (img, 1, i-5)
      }
      imgets (image = img, param = "i_naxis1")
      n1 = int (imgets.value)
      imgets (image = img, param = "i_naxis2")
      n2 = int (imgets.value)
      if (n1 == 800 && n2 == 1200) {
        imgets (image = img, param = "IMAGETYP")
        imagetyp = substr (imgets.value, 1, 8)
        imstatistics (images=img//"[400:500,400:500]",
                      fields="mean,stddev",format-) | scan (mean, stddev)
        if (imagetyp == "twilight" && mean > 6000.0 && mean < 35000.0) {
          if (!HaveTwilights) {
            printf ("%s\n", img, >> TwilightFile)
          }
        } else if (!HaveStars || !HaveFlats) {
          if (mean > 16000.0 && mean < 24000.0 && stddev < 400) {
            if (!HaveFlats) {
              printf ("%s\n", img, >> FlatFile)
            }
          } else {
            if (!HaveStars) {
              printf ("%s\n", img, >> StarFile)
            }
          }
        }
      } else {
        if (!HaveUnknowns) {
          printf ("%s\n", img, >> UnknownFile)
        }
      }
    }
  }

  if (!HaveFlat) {

# Create some temporary files.
  
    t1 = mktemp ("tmp")
    t2 = mktemp ("tmp")
    t3 = mktemp ("tmp")
    printf ("processing flat field images...\n")
    AptProc (dir="./", files="@"//FlatFile, useflat-, bias=0, short-)
    imcombine ("@"//FlatFile//"//p", output=t1, combine="average",
        reject="avsigclip",scale="median",weight="median",mclip+,lsigma=6,
        hsigma=6)
    imglist = FlatFile
    while (fscan (imglist, img) != EOF) {
      i = strlen (img)
      if (substr (img, i-4, i) == ".fits") {
        img = substr (img, 1, i-5)
      }
      imdelete (t2//","//t3, ver-, >& "dev$null")
      img = img//"p"
      imarith (img, "/", t1, t2)
      imsurfit (input=t2, output=t3, xorder=3, yorder=3, type_ou="fit",
          function="spline3", cross+, regions="all")
      imdelete (t2, ver-)   
      imarith (img, "/", t3, img)
      imstatistics (images=t3, fields="min,max", format-) | scan (pmin,pmax)
      percent = 200.0 * (pmax - pmin) / (pmax + pmin)
      printf ("%s has %5.2f%s residual slope\n", img, percent, "%")
    }
    imcombine ("@"//FlatFile//"//p", output=Flat, combine="average",
        reject="avsigclip",scale="none",weight="none",mclip+,lsigma=3,
        hsigma=2)
    imstatistics (images=Flat, fields="mean", format-) | scan (mean)
    imarith (Flat, "/", mean, Flat)
  }

#  AptProc (dir="./", files="@"//StarFile, useflat+, bias=0, 
#      short-, flat=SuperFlat)

    if (!HaveTwilight) {
      printf ("processing twilight images...\n")
    AptProc (dir="./", files="@"//TwilightFile, useflat+, bias=0, 
       short-, flat=Flat)

    imglist = TwilightFile
# Create some temporary files.
  
#goto skip
    t1 = mktemp ("tmp")
    t2 = mktemp ("tmp")
    t3 = mktemp ("tmp")
    NumTwilights = 0;
    while (fscan (imglist, img) != EOF) {
      NumTwilights += 1;
      printf ("%s ", img)
      i = strlen (img)
      if (substr (img, i-4, i) == ".fits") {
        img = substr (img, 1, i-5)
      }
      imdelete (t1//","//t2//","//t3, ver-, >& "dev$null")
      img = img//"p"
      imsurfit (input=img, output=t1, xorder=4, yorder=4, type_ou="fit",
          function="spline3", cross+, regions="all")
      imarith (img, "/", t1, t2)
      imstatistics (images=t2, fields="mean,stddev", format-) | scan (mean,stddev)
      imexpr ("a < "//str(mean+1.5*stddev)//" ? 0 : 1", t3, t2)
      imdelete (img//"m.pl", ver-, >& "dev$null")
      imcopy (input=t3, output=img//"m.pl", >& "dev$null")
      hedit (images=img, fields="BPM", value=img//"m.pl", add+, delete-, verify-, >& "dev$null")
    }

skip:
      imdelete (t1, ver-, >& "dev$null")
      imdelete (t1//".pl", ver-, >& "dev$null")
    AptProc (dir="./", files="@"//TwilightFile, useflat-, bias=0, 
       short-)
      imdelete (Twilight//"m.pl", ver-, >& "dev$null")
  imcombine ("@"//TwilightFile//"//p", output=Twilight, 
         plfile=Twilight//"m.pl",
         combine="average", masktype="goodval", maskval=0, 
         reject="avsigclip",scale="median",weight="median",mclip+,lsigma=6,
        hsigma=3)
      imcopy (Twilight//"m.pl", t1//".pl")
      imdelete (Twilight//"m.pl", ver-, >& "dev$null")
      imexpr ("a != "//str(NumTwilights)//" ? 0 : 1", Twilight//"m.pl",
           t1//".pl")
    }
# imdel a1,a2,a3,a4
# imsurfit im28165p a1 3 3 funct=spline3
# imarith im28165p / a1 a2
# imstat a2
# imexpr "a > 0.988 && a < 1.006 ? 1 : 0" a3 a2
# imarith im28165p * a3 a4
# im28160p,im28161p,im28162p,im28163p,im28164p,im28165p
end
