include <imhdr.h>
include <math.h>
include <mach.h>
include <fset.h>
include <gset.h>

.help aptxymatch Apr99 apt
.ih
NAME
aptxymatch -- 
.ih
USAGE
xc aptxymatch.x
task aptxymatch=aptbin$aptxymatch.e
.ih
PARAMETERS
.ls image
The image within which the reference stars will be located. 
.ls refstars
A text file containing the RA, Dec, ID of the reference stars. The
file has one star on each line with RA and Dec in radians, and
an ID number, all separated by whitespace.
.ls output
The output text file. It contains one line per star, with the following
information: x, y, RA, Dec, and ID. (x,y) is the coordinate of the
star in the image, (RA,Dec) is the position of the star in radians.
ID is the star ID. 
.ih
DESCRIPTION
Given a list of reference stars with known RA, Dec, this routine
attempts to find them in an input image.

This routine is hardwired for the APT, i.e., the plate scale, image
orientation and rotation are built-in. We also use the exposure time
of the image to estimate the minimum signal required to detect a
star, assuming the reference stars are bright (> mag 9).

Since the only remaining unknowns are the (x,y) offset between the
image and the reference stars, the technique used to locate the stars
is to simply conduct a square spiral search, measuring the fraction of
reference stars that appear to correspond to stars in the image.  This
is very robust and fast in practice, and copes nicely with saturated
stars (which are often not detected as stellar if you use a
star-finding algorithm).
.ih
BUGS
The routine always produces an output file, even if the best-fit to
the stars is hopelessly wrong.
.ih
AUTHOR
Author: Michael Ashley / UNSW / 28-Apr-1999 / mcba@newt.phys.unsw.edu.au
.endhelp

task aptxymatch = t_AptXYMatch 

procedure t_AptXYMatch ()

char image[SZ_FNAME]    # input image
char refstars[SZ_FNAME] # the list of reference stars
char output[SZ_FNAME]   # the output text file

define MAX_STARS 1000
define PI 3.14159265359

real ra[MAX_STARS], dec[MAX_STARS]
int  saonum[MAX_STARS], x[MAX_STARS],  y[MAX_STARS]
int  num_stars

pointer imc, bufc
pointer immap(), imgs2r()

int     open(), fscan(), access(), imaccf()
real    imgetr()

int     nx, ny, i
real    mag, rot, a, b, c, d, e, f, ra_centre, dec_centre
real    xcentre, ycentre, cosa, sina, cosd, sind, cotd
real    det, aa, bb, cc, dd, ee, ff, xi, eta
real    exptime
int     ExpectedCounts
int     rfd, ofd
int     best_x, best_y

begin

# Obtain the input image filename, the reference star filename, and
# the output filename.

  call clgstr ("image",     image,    SZ_FNAME)
  call clgstr ("refstars",  refstars, SZ_FNAME)
  call clgstr ("output",    output,   SZ_FNAME)

# Open the input image, and find its dimensions.

  imc = immap (image, READ_ONLY, 0)
  nx  = IM_LEN(imc, 1)
  ny  = IM_LEN(imc, 2)

# Open the refstars file.

  rfd = open (refstars, READ_ONLY, TEXT_FILE)

# Read the refstars into the arrays "ra" and "dec", letting
# "num_stars" equal the total number of stars found.

  num_stars = 0
  while (fscan (rfd) != EOF) {
    num_stars = num_stars + 1
    if (num_stars > MAX_STARS) {
      call printf ("Too many reference stars, please recompile this program\n")
      return
    }
    call gargr (ra[num_stars])
    call gargr (dec[num_stars])
    call gargi (saonum[num_stars])
  }

# Close the refstars file.

  call close (rfd)

# Extract the image centre from the FITS image. The APT images contain
# the RA and Dec of the centre in decimal hours and decimal degress
# respectively. We convert them into radians.

  if (imaccf (imc, "RA") == YES) {
    ra_centre  = imgetr (imc, "RA") * 15 * PI / 180.0
  } else {
    call printf ("RA not found in the image header\n")
    return
  }

  if (imaccf (imc, "DEC") == YES) {
    dec_centre = imgetr (imc, "DEC") * PI / 180.0
  } else {
    call printf ("DEC not found in the image header\n")
    return
  }
  
# Now find the exposure time, in seconds. We use this to estimate
# the number of counts we expect in a reference star.

  if (imaccf (imc, "EXPTIME") == YES) {
    exptime = imgetr (imc, "EXPTIME")
  } else {
    call printf ("EXPTIME not found in the image header\n")
    return
  }

  ExpectedCounts = min (30000.0 * exptime/120.0, 30000.0) 
  
# Initialise the transformation coefficients for the APT.

  xcentre = (1.0 + nx) / 2.0
  ycentre = (1.0 + ny) / 2.0
  mag = 9.413				# The scale in arcsecs/pixel.
  rot = (360.0 -  89.34) * PI / 180.0	# The rotation angle of the CCD.

# These coefficients give (zeta,eta) from (x,y).

  b = mag * cos(rot)
  c = -mag * sin(rot)
  e = -c
  f = b
  a = - b * xcentre - c * ycentre
  d = - e * xcentre - f * ycentre

# Calculate the inverse transform (i.e., from (x,y) to (zeta,eta)).

  det = f*b - e*c
  aa = (c*d - f*a) / det
  bb = f / det
  cc = - c / det
  dd = (e*a - b*d) / det
  ee = - e / det
  ff = b / det

# Calculate "xi" and "eta" for all the reference stars. First we
# precalculate some useful trig functions.

  cosd = cos (dec_centre)
  sind = sin (dec_centre)

  do i = 1, num_stars {
    cosa = cos (ra[i] - ra_centre)
    sina = sin (ra[i] - ra_centre)
    cotd = 1.0 / tan (dec[i])

    xi   = 3600.0 * (180.0/PI) * (cotd * sina) / 
                                 (sind + cosd * cotd * cosa)
    eta  = 3600.0 * (180.0/PI) * (cosd - cotd * sind * cosa) / 
                                 (sind + cotd * cosd * cosa)

    x[i] = nint(aa + bb * xi + cc * eta)
    y[i] = nint(dd + ee * xi + ff * eta)
  }

# Delete the output file, if it exists.

  if (access (output, READ_ONLY, TEXT_FILE) == YES) {
    call delete (output)
  }

# Open the output file.

  ofd = open (output, NEW_FILE, TEXT_FILE)

# Read the image into memory.

  bufc = imgs2r (imc, 1, nx, 1, ny)

# Now search for those stars!

  call process (Memr[bufc], nx, ny, x, y, num_stars, 200, ExpectedCounts,
                best_x, best_y)

# And write the results to the output file.

  do i = 1, num_stars {
    call fprintf (ofd, "%5d %5d %f %f %7d\n")
      call pargi (x[i] + best_x)
      call pargi (y[i] + best_y)
      call pargr (ra[i])
      call pargr (dec[i])
      call pargi (saonum[i])
  }

# Close the output file.

  call close (ofd)

# Close the image.

  call imunmap (imc)

end

procedure process (imc, nx, ny, x, y, num_stars, max_target, ExpectedCounts,
                   best_x, best_y)

# Author: Michael Ashley / UNSW / 28-Apr-1999 / mcba@newt.phys.unsw.edu.au

real imc[nx, ny]
int  nx, ny, num_stars, max_target, ExpectedCounts, best_x, best_y
int  x[num_stars], y[num_stars]

int  sky, xoffset, yoffset, target, length, dx, dy, ddx, ddy
real excellence, max_excellence, xcen, ycen, rad, d0, d1, d2 
int  i, j, xx, yy, num_found, num_missing
real dmax

begin

  xoffset = 0
  yoffset = 0
  target  = 1
  dx      = 1
  ddx     = -1
  dy      = 0
  ddy     = 1

  length  = 0

  max_excellence = -99999

# Find a sky value by examining a row half-way up the image, and
# using the minimum value found on that row. Crude, but probably OK.

  j = max (1, ny / 2)
  sky = 999999
  do i = 1, nx {
    if (imc[i,j] < sky) {
      sky = imc[i,j]
    }
  }

# Now do a square spiral from the nominal centre, until the edge of the
# spiral is "max_target" pixels long.

  while (1 == 1) {

    num_found   = 0
    num_missing = 0
    rad         = 0.0
    xcen        = 0.0
    ycen        = 0.0

    do i = 1, num_stars {

# Find the coordinate of the reference star.

      xx = x[i] + xoffset
      yy = y[i] + yoffset

# Make sure that a 3x3 box centred on the star is wholely within
# the image.

      if ((xx > 1) && (xx < nx) && (yy > 1) && (yy < ny)) {

# Find the maximum data value within the 3x3 box.

        dmax = max (imc[xx-1,yy-1], imc[xx,yy-1], imc[xx+1,yy-1],
                    imc[xx-1,yy],   imc[xx,yy],   imc[xx+1,yy],
                    imc[xx-1,yy+1], imc[xx,yy+1], imc[xx+1,yy+1])

# Have we found a star?

        if ((dmax - sky) > ExpectedCounts) {
          num_found = num_found + 1

          d0 = imc[xx-1,yy-1] + imc[xx-1,yy] + imc[xx-1,yy+1]
          d1 = imc[xx,  yy-1] + imc[xx,  yy] + imc[xx,  yy+1]
          d2 = imc[xx+1,yy-1] + imc[xx+1,yy] + imc[xx+1,yy+1]
          xcen = xcen + (-d0+d2)/(d0+d1+d2 - 3.0 * sky)

          d0 = imc[xx-1,yy-1] + imc[xx,yy-1] + imc[xx+1,yy-1]
          d1 = imc[xx-1,yy]   + imc[xx,yy]   + imc[xx+1,yy]
          d2 = imc[xx-1,yy+1] + imc[xx,yy+1] + imc[xx+1,yy+1]
          ycen = ycen + (-d0+d2)/(d0+d1+d2 - 3.0 * sky)
          
          rad = rad + sqrt(xcen*xcen + ycen*ycen)

        } else {
          num_missing = num_missing + 1
        }
      }
    }

    excellence = (num_found - num_missing) / (1.0 * num_stars)

    if (excellence > max_excellence) {
      max_excellence = excellence
      best_x = xoffset
      best_y = yoffset
    }

# Now calculate the next position in the square spiral.

    length = length + 1
    xoffset = xoffset + dx
    yoffset = yoffset + dy
    if (length == target) {
      length = 0

      dx = dx + ddx
      if (dx == -1) {
        ddx = 1
      } else if (dx == 1) {
        ddx = -1
      }

      if (dx != 0) {
        target = target + 1
        if (target == max_target) {
          break
        }
      }

      dy = dy + ddy
      if (dy == -1) {
        ddy = 1
      } else if (dy == 1) {
        ddy = -1
      }
    }
  }
end
