procedure do_drizzle (images, output, shifts, x_shift, y_shift, fwhmsky, verbose)

#
# Michael Ashley / UNSW / 29-Apr-1998
#
# Given a set of input "images", this script will apply the HST "drizzle"
# technique to combine them to produce an "output" image with twice the
# resolution along each axis. The offsets between the images are stored in
# the text file "shifts". An additional shift of (x_shift, y_shift) is
# added to these shifts. "fwhmsky" is the FWHM of the sky background, in
# counts. 
#
# You must load the following packages first:
#
#    stsdas
#    analysis
#    dither
#
# Type the following command into the IRAF cl to use this program:
#
# task do_drizzle =/usr/users/astro/mcba/apt/do_drizzle.cl
#

string images=""         {prompt="Images to drizzle"}
string output=""         {prompt="Output image"}
string shifts=""         {prompt="File containing the image shifts"}
real   x_shift=0.0       {prompt="Additional shift in x"}
real   y_shift=0.0       {prompt="Additional shift in y"}
real   fwhmsky=10.0      {prompt="FHWM of the sky background"} 
bool   verbose=no        {prompt="Verbose output?"}

struct *IMGLIST
struct *SHIFTS

begin

# Copies of the parameters.

   string Iimages, Ioutput, Ishifts
   real   Ix_shift, Iy_shift, Ifwhmsky
   bool   Iverbose

# Local variables.

   string imgfile, tmpimg, img
   int    nimg, xdim, ydim
   real   skywidth, xs, ys
   bool   first

# Assign positional parameters to local variables.

   Iimages    = images
   Ioutput    = output
   Ishifts    = shifts
   Ix_shift   = x_shift
   Iy_shift   = y_shift
   Ifwhmsky   = fwhmsky
   Iverbose   = verbose

# Restore defaults for all the IRAF routines we use.

   unlearn drizzle
   unlearn sky

   sky.lower   = 0.0
   sky.upper   = 39990.0
   sky.subsky  = yes
   sky.stat    = "mode"
   sky.verbose = Iverbose

# Form temporary filenames.

   imgfile  =  mktemp ("/tmp/mcba")
   tmpimg   =  mktemp ("/tmp/mcba")

   first = yes
   skywidth = 2.0 * Ifwhmsky

# Check existence of input @file.

   if ( (stridx("@",Iimages) == 1) &&
        (!access( substr(Iimages,2,strlen(Iimages))) ) ) {
      print ("DO_DRIZZLE: Input image @file ",Iimages," does not exist",
         " - processing aborted.")
      beep ; goto cleanup
   }

# Check existence of the shifts file.

   if (!access(Ishifts)) {
      print ("DO_DRIZZLE: Can't access shifts file.")
      beep ; goto cleanup
   }

   SHIFTS = Ishifts

# Expand input template into file lists.

   sections (Iimages, option="fullname", > imgfile)
   nimg = sections.nimages
   if (nimg == 0) {
      print ("DO_DRIZZLE: No images in input list - processing aborted.")
      beep ; goto cleanup
   }
   IMGLIST = imgfile

# Scan through each input image,

   while (fscan (IMGLIST,img) != EOF) {
      if ( !access (img) && !access (img//".fits") && !access (img//".imh") ) {
         print ("INITIAL_SHIFTS: Image ",img," does not exist")
         beep ; goto cleanup
      } else {
#         if (stridx('.',img) != 0) {
#            img = substr(img,1,stridx('.',img)-1)
#         }

# Make a copy of the input image.

        imdel  (images=tmpimg, >& "dev$null")
        imcopy (input=img, output=tmpimg, verbose=Iverbose)

# Find the sky and subtract it.

         sky (input=tmpimg//".imh", width=skywidth)

         if (first) {

# Find the size of the first image, and multiply it by two to get
# the size of the output image.

           imgets (image = img, param = "i_naxis1")
           xdim = int (imgets.value) * 4
           imgets (image = img, param = "i_naxis2")
           ydim = int (imgets.value) * 4

# Delete the output file, and the weights file.

           imdel (images=Ioutput,      >& "dev$null")
           imdel (images="W"//Ioutput, >& "dev$null")
           
           first = no
         }

# Find the (x,y) shift for the current image.

         if (fscan(SHIFTS, xs, ys) != EOF) {

# Drizzle the image onto the output image.

            xs += x_shift
            ys += y_shift
            drizzle (data=tmpimg//".imh", outdata=Ioutput//".imh", 
              outweig="W"//Ioutput//".imh",
              in_mask="", wt_scl="exptime", outnx=xdim, outny=ydim,
              pixfrac=0.33, scale=0.25, xsh=xs, ysh=ys)

         } else {
            print ("DO_DRIZZLE: reached premature EOF on shifts file")
            beep ; goto cleanup
         }
     }
   }

cleanup:

# Delete temporary files

   IMGLIST = ""
   SHIFTS  = ""
   delete (imgfile, ver-, >& "dev$null")
   imdel (tmpimg, >& "dev$null")

end
