procedure refine_shifts (images, shifts, xy_stars, verbose)

#
# Michael Ashley / UNSW / 29-Apr-1998
#
# Takes a list of "images", a "shifts" file defining the approximate 
# (x,y) shifts of the images relative to a reference image (which
# is not specified), and a list of (x,y) coordinates of stars ("xy_stars")
# to use to refine the shifts. The "shifts" file is overwritten with the
# improved values, and the original file is saved to a backup file (".bck"
# is prepended).
#
# This script is basically just a wrapper for "imcentroid", which 
# can't produce an output file that is of any direct use.
#
# Type the following command into the IRAF cl to use this program:
#
# task refine_shifts =/usr/users/astro/mcba/apt/refine_shifts.cl
#

string images=""   {prompt="Images to align"}
string shifts=""   {prompt="File of image shifts; will be updated"} 
string xy_stars="" {prompt="File of xy coords of stars to be centroided"}
bool   verbose=no  {prompt="Verbose output?"}

struct *IMGLIST

begin

# Copies of the parameters.

   string Iimages, Ishifts, Ixy_stars
   bool   Iverbose

# Local variables.

   string imgfile, img, tmpfile, tmp2file, shfile
   int    nimg
   bool   first
   real   x, y, x_zero, y_zero
   int    wcs, ix, iy
   struct command
   string key
   imcur  cursor

# Assign positional parameters to local variables.

   Iimages   = images
   Ishifts   = shifts
   Ixy_stars = xy_stars
   Iverbose  = verbose

# Restore defaults for all the IRAF routines we use.

   unlearn imcentroid
   unlearn imcntr
   unlearn sections

# Form temporary filenames.

   imgfile  =  mktemp ("/tmp/mcba")
   tmpfile  =  mktemp ("/tmp/mcba")
   tmp2file =  mktemp ("/tmp/mcba")
   shfile   =  mktemp ("/tmp/mcba")

   first = yes

# Check existence of input @file.

   if ( (stridx("@",Iimages) == 1) &&
        (!access( substr(Iimages,2,strlen(Iimages))) ) ) {
      print ("REFINE_SHIFTS: Input image @file ",Iimages," does not exist",
         " - processing aborted.")
      beep ; goto cleanup
   }

# Expand input template into file lists.

   sections (Iimages, option="fullname", > imgfile)
   nimg = sections.nimages
   if (nimg == 0) {
      print ("REFINE_SHIFTS: No images in input list - processing aborted.")
      beep ; goto cleanup
   }

   IMGLIST = imgfile

# Check existence of shift and xy_stars files.

   if (!access(Ishifts)) {
      print ("REFINE_SHIFTS: Can't access shifts file.")
      beep ; goto cleanup
   }

   if (!access(Ixy_stars)) {
      print ("REFINE_SHIFTS: Can't access xy_stars coordinate file.")
      beep ; goto cleanup
   }

# Get the name of the first image, to use as the reference.

  if (fscan (IMGLIST,img) != EOF) {

# Let imcentroid do all the work.

     imcentroid (input=Iimages, reference=img, coords=Ixy_stars,
           shifts=Ishifts, boxsize=5, verbose+, >& tmpfile)

# Prepare to process the output text file with a perl script that extracts
# the new shifts, and outputs some diagnostic information.

# Create a shell script to run the perl script.
 
     printf ("#!/usr/local/bin/perl\n", >> shfile)
     printf ("/usr/users/apt/iraf/apt/update_shifts.pl %s %s %s\n", 
         Ishifts, tmpfile, tmp2file, >> shfile)

# Run the perl script.

     sh (shfile)

# Make a backup copy of the IShifts file, and then overwrite the 
# Ishifts file with the new centroids.

     delete (Ishifts//".bck",  ver-, >& "dev$null")
     copy (input=Ishifts, output=Ishifts//".bck", verbose=Iverbose)
     printf ("REFINE_SHIFTS: original shifts file backed up to %s\n", 
        Ishifts//".bck")

     delete (Ishifts,  ver-, >& "dev$null")
     copy (input=tmp2file, output=Ishifts, verbose=Iverbose)

   } else {
      print ("REFINE_SHIFTS: Can't find any images")
      beep ; goto cleanup
   }

   cleanup:

# Delete temporary files

   IMGLIST = ""
   delete (imgfile,  ver-, >& "dev$null")
   delete (tmpfile,  ver-, >& "dev$null")
   delete (tmp2file, ver-, >& "dev$null")
   delete (shfile,   ver-, >& "dev$null")

end
